package com.simpledbbook.user;

import java.util.*;

import com.simpledbbook.*;

public class SimpleDBUserTool {
  private static final String AUTH_NAMES = "authorities";
  private static final String PASS_NAME = "password";
  private static final String ENAB_NAME = "enabled";
  private static final String GROUP_NAME = "group";
  private String domainName = "users";
  private boolean enableGroups;
  private SimpleDB sdb;

  public User storeUserByUsername(String username) {
    Item item = sdb.get(domainName, username);
    List<String> auth = null;
    setAuthorities(item);
    String pass = item.getAttribute(PASS_NAME);
    String enabledString = item.getAttribute(ENAB_NAME);
    boolean enabled = enabledString.equals("true");
    User user = new User(username, pass, enabled, auth);
    return user;
  }

  private void setAuthorities(Item item) {
    if (!enableGroups) {
      item.getAttributes(AUTH_NAMES);
    }
    List<String> groupNames = item.getAttributes(GROUP_NAME);
    String query = buildGroupQuery(groupNames);
    sdb.singleAttributeSelect(query, AUTH_NAMES);
  }

  private String buildGroupQuery(List<String> groupNames) {
    String f = "SELECT '%s' FROM `%s` WHERE itemName() IN(%s)";
    String groups = listToQueryString(groupNames);
    return String.format(f, AUTH_NAMES, domainName, groups);
  }

  private String listToQueryString(List<String> groupNames) {
    String result = "";
    for (String group : groupNames) {
      result = result + "," + group;
    }
    return result.substring(1);
  }

  public boolean isEnableGroups() {
    return enableGroups;
  }

  public void setEnableGroups(boolean groups) {
    enableGroups = groups;
  }

  public String getDomainName() {
    return domainName;
  }

  public void setDomainName(String domainName) {
    this.domainName = domainName;
  }

  public void setSimpleDBClient(SimpleDB simpleDb) {
    sdb = simpleDb;
  }
}
