package com.simpledbbook.CH03;

import java.io.*;
import java.util.*;

import com.xerox.amazonws.common.*;
import com.xerox.amazonws.sdb.*;

public class Samples {
   private String accessKeyID;
   private String secretAccessKey;

   {
      try {
         loadCredentialsFromDisk();
      } catch (Exception e) {
         String errorMsg = "credentials not found in $HOME/.awssecret";
         throw new RuntimeException(errorMsg);
      }
   }

   private void loadCredentialsFromDisk() throws Exception {
      Properties props = new Properties();
      String home = System.getProperty("user.home");
      props.load(new FileReader(new File(home, ".awssecret")));
      accessKeyID = props.getProperty("AWS_ACCESS_KEY_ID");
      secretAccessKey = props.getProperty("AWS_SECRET_ACESSS_KEY");
   }
   public static void main(String[] argz) {
      new Samples().invokeAll();
   }

   private void invokeAll() {
      //deleteDomain();
      //createDomain();
      listDomains();
      //putAttributes();
      //metadata();
      //getAttributes();
      //deleteAttributes();
      getAttributes();
      //batchPut();
      //select();
      //selectWithNextToken();
   }

   private void createDomain() {
      SimpleDB simpleDB = new SimpleDB(accessKeyID, secretAccessKey);
      try {
         simpleDB.createDomain("users");
         simpleDB.createDomain("products");
         System.out.println("create domain succeeded");
      } catch (SDBException e) {
         AWSError error = e.getErrors().get(0);
         String code = error.getCode();
         String message = error.getMessage();
         System.err.printf("create domain failed: %s: %s", code, message);
      }
   }

   private void listDomains() {
      boolean secure = true;
      String eu = "sdb.eu-west-1.amazonaws.com";
      SimpleDB simpleDB = new SimpleDB(accessKeyID, secretAccessKey, secure, eu);
      try {
         ListDomainsResult result = simpleDB.listDomains();
         for (Domain domain : result.getDomainList()) {
            System.out.println(domain.getName());
         }
      } catch (SDBException e) {
         AWSError error = e.getErrors().get(0);
         String code = error.getCode();
         String message = error.getMessage();
         System.err.printf("list domain failed: %s: %s", code, message);
      }
   }

   private void deleteDomain() {
      SimpleDB simpleDB = new SimpleDB(accessKeyID, secretAccessKey);
      try {
         simpleDB.deleteDomain("users");
         System.out.println("delete domain succeeded");
      } catch (SDBException e) {
         AWSError error = e.getErrors().get(0);
         String code = error.getCode();
         String message = error.getMessage();
         System.err.printf("delete domain failed: %s: %s", code, message);
      }
   }

   private void metadata() {
      SimpleDB simpleDB = new SimpleDB(accessKeyID, secretAccessKey);
      try {
         Domain d = simpleDB.getDomain("products");
         DomainMetadataResult meta = d.getMetadata();
         System.out.printf("'%s': %d items", d.getName(), meta.getItemCount());
      } catch (SDBException e) {
         AWSError error = e.getErrors().get(0);
         String code = error.getCode();
         String message = error.getMessage();
         System.err.printf("domain metadata failed: %s: %s", code, message);
      }
   }

   private void putAttributes() {
      SimpleDB simpleDB = new SimpleDB(accessKeyID, secretAccessKey);
      boolean replace = false;
      ItemAttribute[] userData = new ItemAttribute[] {
         new ItemAttribute("userid", "Mocky", replace),
         new ItemAttribute("location", "Oswego", replace),
         new ItemAttribute("tags", "SimpleDB", replace),
         new ItemAttribute("tags", "Azure-Table", replace),
         new ItemAttribute("tags", "App-Engine", replace),
      };

      try {
         Domain domain = simpleDB.getDomain("users");
         Item newItem = domain.getItem("1234567");
         newItem.putAttributes(Arrays.asList(userData));
         System.out.println("put attributes succeeded");
      } catch (SDBException e) {
         AWSError error = e.getErrors().get(0);
         String code = error.getCode();
         String message = error.getMessage();
         System.err.printf("put attributes failed: %s: %s", code, message);
      }
   }

   private void getAttributes() {
      SimpleDB simpleDB = new SimpleDB(accessKeyID, secretAccessKey);
      try {
         Domain domain = simpleDB.getDomain("users");
         Item item = domain.getItem("1234567");
         List<String> allAttributes = null;
         Map<String,List<String>> user = item.getAttributesMap(allAttributes);

         String name = user.get("userid").get(0);
         String location = user.get("location").get(0);
         List<String> tags = user.get("tags");
         System.out.printf("%s (%s) follows %s", name, location, tags);

      } catch (SDBException e) {
         AWSError error = e.getErrors().get(0);
         String code = error.getCode();
         String message = error.getMessage();
         System.err.printf("get attributes failed: %s: %s", code, message);
      }
   }

   private void deleteAttributes() {
      SimpleDB simpleDB = new SimpleDB(accessKeyID, secretAccessKey);
      try {
         Item oldItem = simpleDB.getDomain("users").getItem("1234567");
         boolean replace = false;
         ItemAttribute delAttr = new ItemAttribute("tags", "App-Engine", replace);
         oldItem.deleteAttributes(Collections.singletonList(delAttr));
         System.out.println("delete attributes succeeded");
      } catch (SDBException e) {
         AWSError error = e.getErrors().get(0);
         String code = error.getCode();
         String message = error.getMessage();
         System.err.printf("delete attributes failed: %s: %s", code, message);
      }
   }

   private void batchPut() {
      SimpleDB simpleDB = new SimpleDB(accessKeyID, secretAccessKey);

      // a collection to map item name to attributes
      Map<String, List<ItemAttribute>> items;
      items = new HashMap<String, List<ItemAttribute>>();

      items.put("Cigar001", buildRomeoCigar("No. 2", "90"));
      items.put("Cigar002", buildRomeoCigar("Toro", "89"));
      items.put("Cigar003", buildRomeoCigar("Churchill", "94"));
      items.put("Cigar004", buildRomeoCigar("Robusto", "81"));

      try {
         Domain domain = simpleDB.getDomain("products");
         domain.batchPutAttributes(items);
         System.out.println("batch put succeeded");
      } catch (SDBException e) {
         AWSError error = e.getErrors().get(0);
         String code = error.getCode();
         String message = error.getMessage();
         System.err.printf("batch put failed: %s: %s", code, message);
      }
   }

   private List<ItemAttribute> buildRomeoCigar(String make,String rating) {
      boolean replace = false;
      ItemAttribute[] attributes = new ItemAttribute[] {
         new ItemAttribute("ProductType", "Cigar", replace),
         new ItemAttribute("Supplier", "Romeo y Julieta", replace),
         new ItemAttribute("Make", make, replace),
         new ItemAttribute("Rating", rating, replace)
      };
      return Arrays.asList(attributes);
   }

   private void select() {
      SimpleDB simpleDB = new SimpleDB(accessKeyID, secretAccessKey);
      try {
         Domain domain = simpleDB.getDomain("users");
         printSelectResults(domain.selectItems("SELECT * FROM users", null));
      } catch (SDBException e) {
         AWSError error = e.getErrors().get(0);
         String code = error.getCode();
         String message = error.getMessage();
         System.err.printf("select failed: %s: %s", code, message);
      }
   }

   private void printSelectResults(QueryWithAttributesResult result) {
      for (String itemName : result.getItems().keySet()) {
         System.out.println(itemName);
         for (ItemAttribute attr : result.getItems().get(itemName)) {
            System.out.println("\t" + attr.getValue());
         }
      }
   }

   private void selectWithNextToken() {
      String select =
         "SELECT Rating, Make, Supplier" +
         "  FROM products" +
         " WHERE Rating > '85'" +
         "   AND ProductType = 'Cigar'" +
         " ORDER BY Rating DESC" +
         " LIMIT 2";

      SimpleDB simpleDB = new SimpleDB(accessKeyID, secretAccessKey);
      try {
         Domain domain = simpleDB.getDomain("products");
         String nextToken = null;
         QueryWithAttributesResult results = null;
         do {
            results = domain.selectItems(select, nextToken);
            printSelectResults(results);
            nextToken = results.getNextToken();
         } while (nextToken != null);

      } catch (SDBException e) {
         AWSError error = e.getErrors().get(0);
         String code = error.getCode();
         String message = error.getMessage();
         System.err.printf("select failed: %s: %s", code, message);
      }
   }



}
