package com.simpledbbook;

import java.util.*;

public class Item {
  private final List<Attribute> atts = new ArrayList<Attribute>();
  private String name;

  public Item(String itemName) {
    name = itemName;
  }

  public Item(String itemName, List<Attribute> attList) {
    this(itemName);
    addAttributes(attList);
  }

  public String toString() {
    return String.format("\nItem: %s\n   %s", name, atts);
  }

  public String getName() {
    return name;
  }
  
  public void setName(String n) {
    name = n;
  }

  public Attribute[] getAtts() {
    return atts.toArray(new Attribute[atts.size()]);
  }

  public String getAttribute(String name) {
    List<String> values = getAttributes(name);
    return values.isEmpty() ? null: values.get(0);
  }

  public List<String> getAttributes(String name) {
    List<String> result = new ArrayList<String>();
    for (Attribute att : atts) {
      if (att.getName().equals(name)) {
        result.add(att.getValue());
      }
    }
    return result;
  }

  public void addAttributes(List<Attribute> newAtts) {
    atts.addAll(newAtts);
  }

  public void addAttribute(String attrName, String attrValue) {
    atts.add(new Attribute(attrName, attrValue));
  }

  public void addAttribute(String attrName, String attrValue, 
      boolean replace) {
    atts.add(new Attribute(attrName, attrValue, replace));
  }
  
  public void addMultiValuedAttr(String attrName, String... attrVals) {
    for (String value: attrVals) {
      addAttribute(attrName, value);
    }
  }
}