package com.simpledbbook.metadata;

import com.simpledbbook.*;

public class ShardingClient {
  private final SimpleDB sdb;
  private AppConfig config;

  public ShardingClient(AppConfig appConfig) {
    config = appConfig;
    sdb = new SimpleDB(config.getServiceURL());
  }

  public void put(Item item) {
    sdb.put(domainForItemName(item.getName()), item);
  }

  public void putIfNotExists(Item item, String expectedName) {
    String domain = domainForItemName(item.getName());
    sdb.putIfNotExists(domain, item, expectedName);
  }

  public void putIfExists(Item item, String expectedName,
      String expectedValue) {
    String domain = domainForItemName(item.getName());
    sdb.putIfExists(domain, item, expectedName, expectedValue);
  }

  public void delete(Item item) {
    sdb.delete(domainForItemName(item.getName()), item);
  }
  
  public void deleteIfNotExists(Item item, String expectedName) {
    String domain = domainForItemName(item.getName());
    sdb.deleteIfNotExists(domain, item, expectedName);
  }
  
  public void deleteIfExists(Item item, String expectedName,
      String expectedValue) {
    String domain = domainForItemName(item.getName());
    sdb.deleteIfExists(domain, item, expectedName, expectedValue);
  }
  
  public Item get(String itemName, String... atts) {
    return sdb.get(domainForItemName(itemName), itemName, atts);
  }

  public Item getConsistent(String itemName, String... atts) {
    return sdb.get(domainForItemName(itemName), itemName, atts);
  }

  private String domainForItemName(String itemName) {
    int hash = ConsistentHash.hash(itemName);
    return config.getDomain(hash);
  }
}
