package com.simpledbbook.servlet;

import java.io.IOException;

import javax.naming.*;
import javax.servlet.ServletException;
import javax.servlet.http.*;

import com.simpledbbook.SimpleDB;
import com.simpledbbook.tasklist.TaskService;

public abstract class BaseServlet extends HttpServlet {
  private static final long serialVersionUID = 1L;
  private static final int ONE_WEEK = 1000 * 60 * 60 * 24 * 7;
  public static final String USER_DOMAIN;
  public static final String TASK_DOMAIN;
  public static final String SESSION_COOKIE_NAME = "tasksession";
  public static final String USER_COOKIE_NAME = "taskuser";
  protected TaskService app= 
    new TaskService(new SimpleDB(), USER_DOMAIN, TASK_DOMAIN);

  static {
    String userDomain = "users_01";
    String taskDomain = "tasks_01";
    try {
      Context ic = new InitialContext();
      Context context = (Context) ic.lookup("java:comp/env");
      userDomain = (String) context.lookup("userDomain");
      taskDomain = (String) context.lookup("taskDomain");
    } catch (Exception e) {
      e.printStackTrace();
    }
    USER_DOMAIN = userDomain;
    TASK_DOMAIN = taskDomain;
  }

  protected void doPost(HttpServletRequest request,
      HttpServletResponse response) throws ServletException,
      IOException {
    doGet(request, response);
  }

  protected String getSessionCookie(HttpServletRequest request) {
    return getCookie(request, SESSION_COOKIE_NAME);
  }

  protected String getUserCookie(HttpServletRequest request) {
    return getCookie(request, USER_COOKIE_NAME);
  }

  private String getCookie(HttpServletRequest request,
      String cookieName) {
    if (request.getCookies() != null) {
      for (Cookie cookie : request.getCookies()) {
        if (cookie.getName().equals(cookieName)) {
          return cookie.getValue();
        }
      }
    }
    return "";
  }

  protected void deleteSessionCookie(HttpServletResponse response) {
    response.setContentType("text/html");
    Cookie cookie = new Cookie(SESSION_COOKIE_NAME, "");
    cookie.setMaxAge(0);
    cookie.setPath("/");
    response.addCookie(cookie);
  }

  protected void setCookies(HttpServletResponse response,
      String session, String user) {
    response.setContentType("text/html");
    Cookie cookie = new Cookie(SESSION_COOKIE_NAME, session);
    cookie.setMaxAge(ONE_WEEK);
    cookie.setPath("/");
    response.addCookie(cookie);
    Cookie cookie2 = new Cookie(USER_COOKIE_NAME, user);
    cookie2.setMaxAge(ONE_WEEK);
    cookie2.setPath("/");
    response.addCookie(cookie2);
  }

  protected boolean empty(String value) {
    return value == null || "".equals(value.trim());
  }
}