package com.simpledbbook.tasklist;

import java.util.*;

import com.simpledbbook.*;

public class TaskWorkspace {
  public static final String REVISION = "workspaceRevision";
  private static final String NEXT_TASK = "nextTaskId";
  private final String name;
  private Item originalItem;
  private HashMap<String, List<String>> calendar;
  private int nextTaskId = 0;
  private int revision = 0;
  private String sessionId;
  
  public TaskWorkspace(String workspaceName) {
    name = workspaceName;
    calendar = new HashMap<String, List<String>>();
  }
  
  public int getNextTaskId() {
    return nextTaskId;
  }

  public void setNextTaskId(int nextTaskId) {
    this.nextTaskId = nextTaskId;
  }

  public int getWorkspaceRevision() {
    return revision;
  }
  
  public void setWorkspaceRevision(int rev) {
    revision = rev;
  }

  public String getSessionId() {
    return sessionId;
  }

  public void setSessionId(String sessionId) {
    this.sessionId = sessionId;
  }

  public List<String> getDay(String day) {
    List<String> list = calendar.get(day);
    List<String> result = new ArrayList<String>();
    if (list != null) {
      result.addAll(list);
    }
    return result;
  }
  
  public void setDay(String day, List<String> values) {
    calendar.put(day, new ArrayList<String>(values));
  }

  public Item toItem() {
    Item item = new Item(name);
    item.addAttribute(NEXT_TASK, nextTaskId + "", true);
    item.addAttribute(REVISION, revision + "", true);
    item.addAttribute("sessionId", sessionId, true);
    for (Map.Entry<String, List<String>> day: calendar.entrySet()){
      String serializedDay = serializeDay(day.getValue());
      item.addAttribute("date_" + day.getKey(), serializedDay, true);
    }
    return item;
  }
  
  public static TaskWorkspace fromItem(Item item) {
    TaskWorkspace result = new TaskWorkspace(item.getName());
    result.nextTaskId = Integer.parseInt(item.getAttribute(NEXT_TASK));
    result.revision = Integer.parseInt(item.getAttribute(NEXT_TASK));
    result.sessionId = item.getAttribute("sessionId");
    result.originalItem = item;
    result.calendar = toCalendar(item);
    return result;
  }

  private static String serializeDay(List<String> values) {
    StringBuilder result = new StringBuilder();
    for (String s : values) {
      result.append(',').append(s);
    }
    return result.substring(1); // omit the leading ','
  }
  
  private static HashMap<String, List<String>> toCalendar(Item item) {
    HashMap<String, List<String>> result;
    result = new HashMap<String, List<String>>();
    for (Attribute attribute: item.getAtts()) {
      String name = attribute.getName();
      if (name.startsWith("date_")) {
        String date = name.substring(5);
        List<String> tasks = parseTasks(attribute.getValue());
        result.put(date, tasks);
      }
    }
    return result;
  }

  private static List<String> parseTasks(String value) {
    String[] values = value.split(","); 
    return Arrays.asList(values);
  }
  
  public String toString() {
    return originalItem.toString();
  }
}