package com.simpledbbook.tasklist;

import java.util.List;

import com.simpledbbook.SimpleDB;
import com.sun.servicetag.UnauthorizedAccessException;

public class TestUserService {
  private static final String TEST_DOMAIN = "user_test";
  private static final String USERNAME = "MockyUserTest";
  private static final String PASS = "WrkAllDay!";
  private static final String REG = "Registered_Users";
  private static final String TODO = "TODO_Users";
  private static final String TODO_ADMIN = "TODO_Administrators";
  private SimpleDB sdb;
  private UserService userService;

  public static void main(String[] argz) {
    TestUserService test = new TestUserService();
    test.runTests();
  }

  public TestUserService() {
    sdb = new SimpleDB();
    userService = new UserService(sdb, TEST_DOMAIN);
    ensureDomainExists();
  }

  private void ensureDomainExists() {
    List<String> domains = sdb.listDomains();
    if (! domains.contains(TEST_DOMAIN)) {
      sdb.createDomain(TEST_DOMAIN);
    }
  }

  private void runTests() {
    createNewUser();
    authenticateCorrectly(REG, TODO);
    authenticateWrongly(PASS + " ");
    addNewAuthGroup();
    authenticateCorrectly(REG, TODO, TODO_ADMIN);
    removeAuthGroup();
    List<String> groups = authenticateCorrectly(REG, TODO);
    assert !groups.contains(TODO_ADMIN): "Auth group not removed: " + TODO_ADMIN;
    deleteUser();
    authenticateWrongly(PASS);
  }
  private void createNewUser() {
    userService.createUser(USERNAME, PASS, REG, TODO);
    pause();
  }

  private List<String> authenticateCorrectly(String... expectedGroups) {
    List<String> groups = userService.authenticate(USERNAME, PASS);
    for (String expected: expectedGroups) {
      assert groups.contains(expected) : "Auth group not stored: " + expected;
    }
    return groups;
  }

  private void authenticateWrongly(String password) {
    try {
      userService.authenticate(USERNAME, password);
    } catch (UnauthorizedAccessException expected) {
      return;
    }
    assert false : "Bad auth attempt succeeded";
  }

  private void addNewAuthGroup() {
    userService.addAuthGroup(USERNAME, TODO_ADMIN);
    pause();
  }

  private void removeAuthGroup() {
    userService.removeAuthGroup(USERNAME, TODO_ADMIN);
    pause();
  }
  
  private void deleteUser() {
    userService.deleteUser(USERNAME);
    pause();
  }
  
  private void pause() {
    try {
      Thread.sleep(3000);
    } catch (InterruptedException e) {
      e.printStackTrace();
    }
  }
}