package com.simpledbbook.tasklist;

import java.nio.charset.Charset;
import java.security.*;
import java.util.*;

import com.simpledbbook.*;
import com.sun.org.apache.xerces.internal.impl.dv.util.Base64;

public class UserService {
  private static final String SALT = "Salting-Thwarts-Rainbow-Attack";
  private static final Charset UTF_8 = Charset.forName("UTF-8");
  private static final String AUTH_GROUP = "AUTH_GROUP";
  private static final String PASSWORD = "PASSWORD";
  private static final String ENABLED = "ENABLED";
  private MessageDigest MD5;
  private SimpleDB sdb;
  private String domainName;
  
  public UserService(SimpleDB simpleDB, String domain) {
    sdb = simpleDB;
    domainName = domain;
    try {
      MD5 = MessageDigest.getInstance("MD5");
    } catch (NoSuchAlgorithmException cannotHappen) {}
  }

  public String createUser(String name, String pass, String... group){
    Item item = new Item(name);
    item.addAttribute(PASSWORD, hash(pass));
    item.addAttribute(ENABLED, Boolean.TRUE.toString());
    item.addMultiValuedAttr(AUTH_GROUP, group);
    try {
      sdb.putIfNotExists(domainName, item, PASSWORD);
    } catch (RuntimeException e) { // condition failed
      throw new UserNameUnavailable();
    }
    return item.getName();
  }
  
  public void deleteUser(String name) {
    sdb.delete(domainName, new Item(name));
  }
  
  private String hash(String pass) {
    byte[] salted = (SALT + pass).getBytes(UTF_8);
    byte[] hash;
    synchronized (MD5) {
      hash = MD5.digest(salted);
    }
    return Base64.encode(hash);
  }

  public List<String> authenticate(String userName, String password) {
    Item user = sdb.get(domainName, userName);
    String suppliedCredential = hash(password);
    String storedCredential = user.getAttribute(PASSWORD);
    if (! suppliedCredential.equals(storedCredential)) {
      throw new AuthenticationFailure();
    }
    return user.getAttributes(AUTH_GROUP);
  }
  
  public void addAuthGroup(String userName, String group) {
    Item item = new Item(userName);
    item.addMultiValuedAttr(AUTH_GROUP, group);
    sdb.put(domainName, item);
  }
  
  public void removeAuthGroup(String userName, String group) {
    Item item = new Item(userName);
    item.addMultiValuedAttr(AUTH_GROUP, group);
    sdb.delete(domainName, item);
  }
  
  public List<String> listUsers(String group) {
    String query = "SELECT itemName() FROM `" + domainName +
      "` WHERE " + AUTH_GROUP + " = '" + group + "'";
    List<Item> items = sdb.selectWithRealLimit(query, 100000);
    List<String> result = new ArrayList<String>();
    for (Item item: items) {
      result.add(item.getName());
    }
    return result;
  }
}
